' NavSync:
' Revision: v1.2.1 (vb)
' (c) 2016 steje
'
' This is a script for written for Directory Opus v11.
' See http://www.gpsoft.com.au/redirect.asp?page=scripts for development information.

' =====================================================================================
' ============================   SCRIPT COMMAND: NavSync   ============================
' ======================   EVENT HANDLER: OnBeforeFolderChange   ======================
'
' The purpose of this script is to provide an alternative to the built-in NavLock
' feature in Opus. The differences between this script and NavLock are as follows:
'
' Loss of sync notification:
' --------------------------
'   - NavLock throws up a slide-down notification at the top of the file displays with
'     useful options for handling loss-of-sync situations, but I find that method far
'     too disruptive to my use of the file display.
'   - NavSync instead provides several user-configurable options that allow control
'     over how the script notifies you of a loss-of-sync condition:
'
'     The Script Configuration option that controls this is SYNC_LOSS_PROFILE, and it
'     allows the following selections:
'
'     1. NONE:      No action is taken when loss-of-sync occurs.
'
'     2. CUSTOM:    When set, the script will look for the Opus Command values set
'                   by the user in each of the SYNC_LOST_ACTION and SYNC_REC_ACTION
'                   script config options.
'
'     3. FORMAT:    The script will change the folder format of the linked tab(s) that
'                   lost sync to a user-defined favorite format that must be called
'                   'navsync'. The user must first create this format themselves for
'                   this option to work.
'
'     4. RESYNC:    The script will attempt to automatically create the missing
'                   folder path(s) to automatically recover (really, prevent) loss
'                   of sync... If the attempt fails, no obvious action is taken
'                   other than any errors that might be thrown by the failed
'                   attempt to create the folder.
'
'                   NOTE:  This behavior can also be triggered on-demand by the
'                   NavSync RESYNC command.
'
' Multiple Lister support:
' -----------------------
'   - NavLock only works with two FILE DISPLAYS in the same lister.
'   - NavSync works with tabs across multiple LISTERS in order to make Navlock-like
'     functionality available to users who want to sync navigation across listers.
'     This is an obvious nice-to-have for those of us that work with multiple monitors.
'
' Switching between tabs:
' -----------------------
'   - NavLock (once enabled) establishes a link between the current paths in each file
'     display. In cases where you might have more than just TWO sets of similar folder
'     structures, switching to one of them in another tab once you've navigated down
'     the folder tree in the initial TWO synced tabs makes it cumbersome to true up
'     the THIRD tab with either of the other two without seeing alot of the crazy out
'     of sync notifications described above, as well as some cases where you end up
'     with the exact same paths in both tabs, which I don't think I ever wantto have
'     happen :-) ...
'   - NavSync allows the user to switch tabs similarly to how linked slave tabs work
'     where the synchronized navigation takes place explicitly between the linked
'     tabs as opposed to whatever the current source and destination tabs are (which
'     could be different than the tabs that are actually linked).
'
'     FUTURE == optionally handle different tab situations in either of the following
'     ways:
'
'     1. Automatically attempt to position the newly activated folder tab to the
'        same relative position as the folder tab which was just left (the one
'        that was just deactivated, in the same file display).
'
'        If the path is not valid, optionally prompt user to perform the 'resync'
'        function in order to create the missing folder path(s) in the newly
'        activated tab.
'
'     2. Automatically 'reset' the tab in the other file display back to the initial
'        folder path it was in when NavSync was turned on. This is to provide an
'        easy way of enabling NavSync between one of the originally synced tabs and
'        the newly activated one - assuming the new tab has been opened to a common
'        enough starting point with a similar folder structure as the other tab
'        in the original synced pair of tabs...
'
' FUTURE == Multiple (more than 2) tab support:
' ---------------------------------------------
'   - NavLock only works with TWO tabs (again, in the same lister).
'   - NavSync WILL support link and sync of up to FOUR tabs, open across up to FOUR listers
'     by creating a vector of linked tab ID's in the navsync environment variable of each
'     tab. A more common use case than FOUR separate listers is probably two dual-display
'     listers...
'
' =====================================================================================

Option Explicit
Dim SCRIPT_DEBUG
SCRIPT_DEBUG = False


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Called by Directory Opus to initialize the script
Function OnInit(ByRef initData)

   DOpus.Output("Initializing...")

   ' Provide basic information about the Script
   initData.min_version = "11.16"
   initData.name = "NavSync"
   initData.desc = "The NavSync script implements both an Event Handler and a Script " &_
                   "Command to control the synchronization of folder changes between " &_
                   "file displays similar to the native NavLock feature, but across " &_
                   "multiple listers and monitors as well as user-configurable options" &_
                   "for what happens when a loss of sync situation occurs."
   initData.copyright = "(c) 2015 steje"
   initData.version = "v1.2.1 (vb)"
   initData.default_enable = True

   '////////////////////////////////////////////////
   ' The following options are user configurable via the Configure button under the scripts listing in Prefs
   '////////////////////////////////////////////////
   ' Set DEBUG flag to True in order to enable logging messages to the Opus Output Window
   initData.config.DEBUG = False
   '////////////////////////////////////////////////
   ' Set DEBUG_CLEAR flag to True in order to clear log messages from the Opus Output Window between script runs
   initData.config.DEBUG_CLEAR = False
   '////////////////////////////////////////////////
   ' Set SYNC_LOSS_PROFILE in order to control behavior when sync has been lost
   initData.config.SYNC_LOSS_PROFILE = DOpus.NewVector(0, "NONE", "CUSTOM", "FORMAT", "RESYNC")
   '////////////////////////////////////////////////
   ' Set SYNC_OK_FORMAT in order to control behavior when sync has been lost
   'initData.config.SYNC_OK_FORMAT = False
   '////////////////////////////////////////////////
   ' Set SYNC_LOST_ACTION string to a command that the script should run when sync has been lost
   initData.config.SYNC_LOST_ACTION = ""
   '////////////////////////////////////////////////
   ' Set SYNC_REC_ACTION string to a command that the script should run when sync has been recovered after having been lost
   initData.config.SYNC_REC_ACTION = ""
   '////////////////////////////////////////////////
   ' Set descriptions for all script options
   initData.config_desc = DOpus.NewMap("DEBUG", "Set this option to True to enable logging to the Opus script log.", _
                                       "DEBUG_CLEAR", "Set this option to True clear messages logged to the Opus script log between script runs.", _
                                       "SYNC_LOSS_PROFILE", "Set this option to control the notification style when loss-of-sync conditions occur.", _
                                       "SYNC_LOST_ACTION", "Set this option to a custom command you want the script to run when a loss-of-sync condition occurs. Works when SYNC_LOSS_PROFILE is set to CUSTOM.", _
                                       "SYNC_REC_ACTION", "Set this option to a custom command you want the script to run when a the script recovers from a loss-of-sync condition. Works when SYNC_LOSS_PROFILE is set to CUSTOM.")

                                       '"SYNC_OK_FORMAT", "Set this option to True if SYNC_LOSS_PROFILE is set to FORMAT, and you want to apply a format called 'navsync_ok' when tabs are properly synced.", _

   ' Initialize the command that this script adds
   Dim cmd
   Set cmd = initData.AddCommand()
   cmd.name = "NavSync"
   cmd.icon = "navlock"
   cmd.method = "OnNavSync"
   cmd.desc = "The NavSync command can be used to control the NavSync script."
   cmd.label = "NavSync"
   cmd.template = "ON/S,OFF/S,TOGGLE/S,RESET/S,RESYNC/S,STATS/S"

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Called before the folder has been changed
Function OnBeforeFolderChange(ByRef beforeFolderChangeData)
   Dim syncRecFormatOk
   Dim objSrcTab, objSrcTabCmd, objSrcTabVar, objSrcTabVarNew
   Dim objDstTab, objDstTabCmd, objDstTabVar, objDstTabVarNew
   Dim strNewRelPath, strNewFullPath, strSyncLossProfile, strSyncLostAction, strSyncRecAction, strCtrlFlag
   Const vbQuote = """"

   OnBeforeFolderChange = False
   If Not (beforeFolderChangeData.initial) Then
      Set objSrcTabCmd = DOpus.NewCommand
      If (objSrcTabCmd.IsSet("$glob:SCRIPT_DEBUG")) Then SCRIPT_DEBUG = True
      If ((Script.config.DEBUG) Or (SCRIPT_DEBUG)) And (Script.config.DEBUG_CLEAR) Then DOpus.ClearOutput
      'logMsg("OnBeforeFolderChange()")

      Set objSrcTab = beforeFolderChangeData.tab
      objSrcTab.Update
      objSrcTabCmd.SetSourceTab(objSrcTab)
      Set objDstTabCmd = DOpus.NewCommand

      strCtrlFlag = "OnBeforeFolderChange"
      If Not (executeScript(strCtrlFlag, beforeFolderChangeData.path, beforeFolderChangeData.action, objSrcTab, objDstTab, objSrcTabCmd, objDstTabCmd, objSrcTabVar, objDstTabVar, objSrcTabVarNew, objDstTabVarNew, strNewRelPath, strNewFullPath, strSyncLossProfile, strSyncLostAction, strSyncRecAction, syncRecFormatOk)) Then
         ' Not much to do with current layout of script - all setup work handled in executeScript()
      End If

      Set objSrcTabVar = Nothing
      Set objDstTabVar = Nothing
      Set objSrcTabCmd = Nothing
      Set objDstTabCmd = Nothing
      Set objSrcTab = Nothing
      Set objDstTab = Nothing

   End If

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Implement the NavSync command
Function OnNavSync(ByRef commandData)
   Dim syncRecFormatOk
   Dim objSrcTab, objSrcTabCmd, objSrcTabVar, objSrcTabVarNew
   Dim objDstTab, objDstTabCmd, objDstTabVar, objDstTabVarNew
   Dim objDlg
   Dim strNewRelPath, strNewFullPath, strSyncLossProfile, strSyncLostAction, strSyncRecAction, strAction, strCtrlFlag
   Const vbQuote = """"

   Set objSrcTabCmd = commandData.func.command
   If (objSrcTabCmd.IsSet("$glob:SCRIPT_DEBUG")) Then SCRIPT_DEBUG = True
   If ((Script.config.DEBUG) Or (SCRIPT_DEBUG)) And (Script.config.DEBUG_CLEAR) Then DOpus.ClearOutput
   logMsg("----------------------------------------------------------------")

   Set objSrcTab = commandData.func.sourcetab
   objSrcTab.Update
   ' objSrcTabCmd already has SourceTab set from commandData
   Set objDstTabCmd = DOpus.NewCommand

   '////////////////////////////////////////////////
   ' Check for the ON option
   If (commandData.func.args.got_arg.on) Then
      logMsg("NavSync:  On")
      Call NavSync_On(commandData, objSrcTab, objDstTab, objSrcTabCmd, objDstTabCmd, objSrcTabVarNew, objDstTabVarNew, syncRecFormatOk)

   '////////////////////////////////////////////////
   ' Check for the OFF option
   ElseIf (commandData.func.args.got_arg.off) Then
      logMsg("NavSync:  Off")
      logMsg("----------------------------------------------------------------")
      Call NavSync_Off(objSrcTab, objDstTab, objSrcTabCmd, objDstTabCmd, objSrcTabVar, objDstTabVar, strSyncLossProfile, strSyncLostAction, strSyncRecAction, syncRecFormatOk)

   '////////////////////////////////////////////////
   ' Check for the TOGGLE option
   ElseIf (commandData.func.args.got_arg.toggle) Then
      If Not (objSrcTab.vars.Exists("navsync")) Then
         logMsg("NavSync:  TOGGLE ON")
         Call NavSync_On(commandData, objSrcTab, objDstTab, objSrcTabCmd, objDstTabCmd, objSrcTabVarNew, objDstTabVarNew, syncRecFormatOk)
      Else
         logMsg("NavSync:  TOGGLE OFF")
         logMsg("----------------------------------------------------------------")
         Call NavSync_Off(objSrcTab, objDstTab, objSrcTabCmd, objDstTabCmd, objSrcTabVar, objDstTabVar, strSyncLossProfile, strSyncLostAction, strSyncRecAction, syncRecFormatOk)
      End If

   '////////////////////////////////////////////////
   ' Check for the RESET option
   ElseIf (commandData.func.args.got_arg.reset) Then
      logMsg("NavSync:  RESET")
      strCtrlFlag = "RESET"
      strAction = ""
      If Not (executeScript(strCtrlFlag, objSrcTab.path, strAction, objSrcTab, objDstTab, objSrcTabCmd, objDstTabCmd, objSrcTabVar, objDstTabVar, objSrcTabVarNew, objDstTabVarNew, strNewRelPath, strNewFullPath, strSyncLossProfile, strSyncLostAction, strSyncRecAction, syncRecFormatOk)) Then

         Set objDlg = objSrcTab.Dlg
         objDlg.icon = "warning"
         objDlg.message = "Unable to RESET NavSync paths!"
         objDlg.title = "NavSync:  RESET Failure"
         logMsg("----------------------------------------------------------------")
         logMsg("Unable to RESET NavSync paths!")
         objDlg.Show
         Set objDlg = Nothing
      End If

   '////////////////////////////////////////////////
   ' Check for the RESYNC option
   ElseIf (commandData.func.args.got_arg.resync) Then
      logMsg("NavSync:  RESYNC")
      strCtrlFlag = "RESYNC"
      strAction = ""
      If Not (executeScript(strCtrlFlag, objSrcTab.path, strAction, objSrcTab, objDstTab, objSrcTabCmd, objDstTabCmd, objSrcTabVar, objDstTabVar, objSrcTabVarNew, objDstTabVarNew, strNewRelPath, strNewFullPath, strSyncLossProfile, strSyncLostAction, strSyncRecAction, syncRecFormatOk)) Then

         Set objDlg = objSrcTab.Dlg
         objDlg.icon = "warning"
         objDlg.message = "Unable to RESYNC NavSync paths!"
         objDlg.title = "NavSync:  RESYNC Failure"
         logMsg("----------------------------------------------------------------")
         logMsg("Unable to RESYNC NavSync paths!")
         objDlg.Show
         Set objDlg = Nothing
      End If

   '////////////////////////////////////////////////
   ' Check for the STATS option
   ElseIf (commandData.func.args.got_arg.stats) Then
      If Not (SCRIPT_DEBUG) Then
         SCRIPT_DEBUG = True
         logMsg("NavSync:  STATS")
         logStats(objSrcTab)
         SCRIPT_DEBUG = False
      Else
         logMsg("NavSync:  STATS")
         logStats(objSrcTab)
      End If
   End If

   logMsg("")

   Set objSrcTabVarNew = Nothing
   Set objDstTabVarNew = Nothing
   Set objSrcTabVar = Nothing
   Set objDstTabVar = Nothing
   Set objSrcTabCmd = Nothing
   Set objDstTabCmd = Nothing
   Set objSrcTab = Nothing
   Set objDstTab = Nothing

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Check for various things that determine if the script should run or not
Function executeScript(ByRef strCtrlFlag, ByRef strNewPath, ByRef strAction, ByRef objSrcTab, ByRef objDstTab, ByRef objSrcTabCmd, ByRef objDstTabCmd, ByRef objSrcTabVar, ByRef objDstTabVar, ByRef objSrcTabVarNew, ByRef objDstTabVarNew, ByRef strNewRelPath, ByRef strNewFullPath, ByRef strSyncLossProfile, ByRef strSyncLostAction, ByRef strSyncRecAction, ByRef syncRecFormatOk)

   'logMsg("executeScript()")

   executeScript = False
   ' Check if navsync is enabled for the current source tab
   If (objSrcTab.vars.Exists("navsync")) Then
      Set objSrcTabVar = objSrcTab.vars("navsync")
      ' Check to see if the current SrcTab has a valid DstTab link... in the future I'll add
      ' an optional function call here to open a new tab and sync/resync it to the source.
      If (getLinkedTab(objSrcTab, objDstTab)) Then
         If (objDstTab.vars.Exists("navsync")) Then
            executeScript = True

            Call getSyncLossActions(strSyncLossProfile, strSyncLostAction, strSyncRecAction, syncRecFormatOk)
            objDstTabCmd.SetSourceTab(objDstTab)
            Set objDstTabVar = objDstTab.vars("navsync")
            Call getNavSyncVars(objSrcTabVar, objSrcTabVarNew)
            Call getNavSyncVars(objDstTabVar, objDstTabVarNew)

            If (strCtrlFlag = "OnBeforeFolderChange") Then
               logMsg("----------------------------------------------------------------")
               logMsg("Folder change detected:")
            End If

            logMsg("----------------------------------------------------------------")
            logMsg("Current SrcPath:   " & DOpus.FSUtil.DisplayName(strNewPath))
            logMsg("Current DstPath:   " & DOpus.FSUtil.DisplayName(objDstTab.path))

            ' Check source path against InitPath path... store the difference in strNewRelPath
            If (strCtrlFlag = "RESET") Then
               Call NavSync_Reset(objSrcTab, objDstTab, objSrcTabCmd, objDstTabCmd, objSrcTabVar, objDstTabVar, objSrcTabVarNew, objDstTabVarNew, strSyncLossProfile, strSyncLostAction, strSyncRecAction, syncRecFormatOk)

            ElseIf (getRelativePath(objSrcTabVar("InitPath"), strNewPath, strNewRelPath, 0, "")) Then
               strNewFullPath = objDstTabVar("InitPath") & strNewRelPath

               ' Check if strNewFullPath exists or not
               If (DOpus.FSUtil.Exists(DOpus.FSUtil.DisplayName(strNewFullPath))) Then
                  ' Depending if the strNewFullPath exists or not, run the sync-loss recovery action if
                  ' necessary BEFORE changing folders. This is the scripts way of hedging bets that if
                  ' the folder path does in fact exist, so should reading the same path. If it doesn't
                  ' we'll catch it later anyway and rerun sync-loss action...
                  If Not (objSrcTabVar("SyncState")) And (strSyncRecAction <> "") Then objSrcTabCmd.RunCommand(strSyncRecAction)
                  If Not (objDstTabVar("SyncState")) And (strSyncRecAction <> "") Then objDstTabCmd.RunCommand(strSyncRecAction)
                  Call NavSync_Go(strNewPath, objSrcTab, objDstTab, objSrcTabCmd, objDstTabCmd, objSrcTabVar, objDstTabVar, objSrcTabVarNew, objDstTabVarNew, strNewFullPath, strSyncLostAction, strSyncRecAction, strAction, syncRecFormatOk)
               ElseIf ((strSyncLossProfile = "RESYNC") Or (strCtrlFlag = "RESYNC")) Then
                  Call NavSync_Resync(strNewPath, objSrcTab, objDstTab, objSrcTabCmd, objDstTabCmd, objSrcTabVar, objDstTabVar, objSrcTabVarNew, objDstTabVarNew, strNewFullPath, strNewRelPath, strSyncLostAction, strSyncRecAction, syncRecFormatOk)
               Else
                  If (strSyncLostAction <> "") Then objDstTabCmd.RunCommand(strSyncLostAction)
                  logMsg("  Path does NOT exist:  " & DOpus.FSUtil.DisplayName(strNewFullPath))
                  logMsg("Sync has been lost!")
                  'objSrcTabVarNew("SyncState") = False
                  objDstTabVarNew("SyncState") = False

                  objSrcTab.vars.Set("navsync") = objSrcTabVarNew
                  objDstTab.vars.Set("navsync") = objDstTabVarNew
               End If
            Else
               If (strSyncLostAction <> "") Then objDstTabCmd.RunCommand(strSyncLostAction)
               logMsg("  The InitPath has been lost")
               logMsg("DstTab path out of sync!")
               'objSrcTabVarNew("SyncState") = False
               objDstTabVarNew("SyncState") = False

               objSrcTab.vars.Set("navsync") = objSrcTabVarNew
               objDstTab.vars.Set("navsync") = objDstTabVarNew
            End If

            If (strCtrlFlag = "RESYNC") Or (strCtrlFlag = "RESET") Then
               logStats(objSrcTab)
            Else
               logMsg("")
            End If
         Else
            logMsg("The tab which the current Srctab was synced with no longer has a valid navsync environment!")
         End If
      Else
         ' FUTURE == Add an option to just open a new tab to the old LinkedTabs' InitPath
         ' and resetting NavSync that way so it's valid again. Would have to adjust the
         ' source tabs environment to point to the new LinkedTabs' ID.
         logMsg("The tab which the current Srctab was synced with is no longer open!")
      End If
   Else
      logMsg("NavSync NOT enabled for current SrcTab")
   End If

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Check if the tab ID passed to this function exists in the current or any other lister
Function checkTabExists(ByRef objTabID, ByRef objLister)
   Dim x, y
   'logMsg("checkTabExists()")

   checkTabExists = False
   ' First check if the tab passed to this function exists in the source lister passed to this function
   For Each x in (objLister.tabs)
      If (x = objTabID) Then
         checkTabExists = True
         Exit Function
      End If
   Next
   ' Then check if the tab passed to this function exists in any other listers
   For Each y in (DOpus.listers)
      If Not (y = objLister) Then
         For Each x in (y.tabs)
            If (x = objTabID) Then
               checkTabExists = True
               Exit Function
            End If
         Next
      End If
   Next

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' If the SrcTab passed to this function has a valid linked tab, assign the linked tab ID to DstTab
Function getLinkedTab(ByRef objSrcTab, ByRef objDstTab)
   Dim objSrcTabVar
   'logMsg("getLinkedTab()")

   getLinkedTab = False
   Set objSrcTabVar = objSrcTab.vars("navsync")
   If Not (checkTabExists(objSrcTabVar("LinkedTabID"), objSrcTab.lister)) Then
      logMsg("The tab which the current Srctab was synced with is no longer open! Disabling NavSync...")
      Set objSrcTabVar = Nothing
      Set objDstTab = Nothing
      Exit Function
   Else
      Set objDstTab = objSrcTabVar("LinkedTabID")
      objDstTab.Update
      getLinkedTab = True
   End If
   Set objSrcTabVar = Nothing

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' If the Tab passed to this function has a valid environment variable, create a copy of it for use in resetting it
Function getNavSyncVars(ByRef objTabVar, ByRef objTabVarNew)
   Dim DOpusFactory
   'logMsg("getNavSyncVars()")

   Set DOpusFactory = DOpus.Create
   Set objTabVarNew = DOpusFactory.Map("LinkedTabID", objTabVar("LinkedTabID"), "SyncState", objTabVar("SyncState"), "InitPath", objTabVar("InitPath"), "SyncPath", objTabVar("SyncPath"))
   Set DOpusFactory = Nothing

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' If the current path is a child directory of the initial path set strPath
Function getRelativePath(ByRef strInitPath, ByRef strNewPath, ByRef strRelPath, ByRef intOffset, ByRef strOffset)
   Dim objTabVar
   'logMsg("getRelativePath()")

   strRelPath = ""
   ' Check if the new folder that the source tab was changed to is still somewhere within
   ' the initial directory structure (SyncPath) that was open when navsync was enabled.
   If (InStr(DOpus.FSUtil.DisplayName(strNewPath), DOpus.FSUtil.DisplayName(strInitPath)) > 0) Then
      If Not (strNewPath = strInitPath) Then
         strRelPath = Mid(strNewPath, (Len(strInitPath) + 1 + intOffset))
         logMsg(strOffset & "Relative SrcPath:  " & strRelPath)
      Else
         strRelPath = ""
         logMsg(strOffset & "Current path same as InitPath")
      End If
      getRelativePath = True
   Else
      ' We're in a completely different directory structure than that which was originally
      ' synced. Need code here to handle like any loss-of-sync condition, and make caller
      ' conditional.
      strRelPath = ""
      getRelativePath = False
   End If

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Set the variables that control the behavior when loss-of-sync conditions occur
Function getSyncLossActions(ByRef strSyncLossProfile, ByRef strSyncLostAction, ByRef strSyncRecAction, ByRef syncRecFormatOk)

   'logMsg("getSyncLossActions")

   strSyncLossProfile = ""
   strSyncLostAction = ""
   strSyncRecAction = ""
   Select Case Script.config.SYNC_LOSS_PROFILE
      Case 0
         'logMsg("SYNC_LOSS_PROFILE = NONE")
         strSyncLossProfile = "NONE"
      Case 1
         'logMsg("SYNC_LOSS_PROFILE = CUSTOM")
         If ((Script.config.SYNC_LOST_ACTION = "") Or (Script.config.SYNC_REC_ACTION = "")) Then
            strSyncLossProfile = "NONE"
         Else
            strSyncLossProfile = "CUSTOM"
            strSyncLostAction = Script.config.SYNC_LOST_ACTION
            strSyncRecAction = Script.config.SYNC_REC_ACTION
         End If
      Case 2
         'logMsg("SYNC_LOSS_PROFILE = FORMAT")
         strSyncLossProfile = "FORMAT"
         strSyncLostAction = "set format navsync"
         strSyncRecAction = "set format !folder"
         'If (Script.config.SYNC_OK_FORMAT) Then syncRecFormatOk = True
         syncRecFormatOk = False
      Case 3
         'logMsg("SYNC_LOSS_PROFILE = RESYNC")
         strSyncLossProfile = "RESYNC"
   End Select

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Shared function for OnafterFolderChange and OnNavSync RESYNC operations to change the DstTab to the new path
Function NavSync_Go(ByRef strNewPath, ByRef objSrcTab, ByRef objDstTab, ByRef objSrcTabCmd, ByRef objDstTabCmd, ByRef objSrcTabVar, ByRef objDstTabVar, ByRef objSrcTabVarNew, ByRef objDstTabVarNew, ByRef strNewFullPath, ByRef strSyncLostAction, ByRef strSyncRecAction, ByRef strAction, ByRef syncRecFormatOk)
   Const vbQuote = """"

   logMsg("  Path exists:     " & DOpus.FSUtil.DisplayName(strNewFullPath))
   logMsg("  Action:          " & strAction)

   ' Only change the DstTab to the new path if it's not already the current path. Use regular Go commands
   ' with an explicit PATH argument, except for parent, back, and forward actions. For those actions, run
   ' the same action in the DstTab as was used in the source as long as we are not currently out of sync.
   If Not (DOpus.FSUtil.ComparePath(DOpus.FSUtil.DisplayName(objDstTab.path), DOpus.FSUtil.DisplayName(strNewFullPath))) Then
      If ((strAction = "normal") Or (strAction = "dblclk") Or (strAction = "") Or Not (objSrcTabVar("SyncState")) Or Not (objDstTabVar("SyncState"))) Then
         'objSrcTabCmd.RunCommand("Go " & vbQuote & strNewFullPath & vbQuote & " NOSCRIPT")
         objDstTabCmd.RunCommand("Go " & vbQuote & strNewFullPath & vbQuote & " NOSCRIPT")
      ElseIf (strAction = "parent") Then
         'objSrcTabCmd.RunCommand("Go UP BACK NOSCRIPT")
         objDstTabCmd.RunCommand("Go UP BACK NOSCRIPT")
      Else
         'objSrcTabCmd.RunCommand("Go " & strAction & " NOSCRIPT")
         objDstTabCmd.RunCommand("Go " & strAction & " NOSCRIPT")
      End If

      ' Depending if the DstTab folder change worked or not, adjust environment variables accordingly
      If (objDstTabCmd.results.result = 1) Then
         logMsg("DstTab path change OK!")
         If Not (objSrcTabVar("SyncState")) And (strSyncRecAction <> "") Then objSrcTabCmd.RunCommand(strSyncRecAction)
         If Not (objDstTabVar("SyncState")) And (strSyncRecAction <> "") Then objDstTabCmd.RunCommand(strSyncRecAction)
         ' If the SYNC_OK_FORMAT option is enabled, set the 'navsync_ok' format for both tabs
         If (syncRecFormatOk) Then
            objSrcTabCmd.RunCommand("set format navsync_ok")
            objDstTabCmd.RunCommand("set format navsync_ok")
         End If
         objSrcTabVarNew("SyncState") = True
         objDstTabVarNew("SyncState") = True
         objSrcTabVarNew("SyncPath") = strNewPath
         objDstTabVarNew("SyncPath") = strNewFullPath

         objSrcTab.vars.Set("navsync") = objSrcTabVarNew
         objDstTab.vars.Set("navsync") = objDstTabVarNew
      Else
         If (strSyncLostAction <> "") Then objDstTabCmd.RunCommand(strSyncLostAction)
         logMsg("DstTab path change FAILED!")
         'objSrcTabVarNew("SyncState") = False
         objDstTabVarNew("SyncState") = False

         objSrcTab.vars.Set("navsync") = objSrcTabVarNew
         objDstTab.vars.Set("navsync") = objDstTabVarNew
      End If
   Else
      logMsg("DstTab path already in sync...")
      If Not (objSrcTabVar("SyncState")) And (strSyncRecAction <> "") Then objSrcTabCmd.RunCommand(strSyncRecAction)
      If Not (objDstTabVar("SyncState")) And (strSyncRecAction <> "") Then objDstTabCmd.RunCommand(strSyncRecAction)
      ' If the SYNC_OK_FORMAT option is enabled, set the 'navsync_ok' format for both tabs
      If (syncRecFormatOk) Then
         objSrcTabCmd.RunCommand("set format navsync_ok")
         objDstTabCmd.RunCommand("set format navsync_ok")
      End If
      objSrcTabVarNew("SyncState") = True
      objDstTabVarNew("SyncState") = True
      objSrcTabVarNew("SyncPath") = strNewPath
      objDstTabVarNew("SyncPath") = strNewFullPath

      objSrcTab.vars.Set("navsync") = objSrcTabVarNew
      objDstTab.vars.Set("navsync") = objDstTabVarNew
   End If

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Turn NavSync OFF by deleting the environment variables for the tab ID passed to this function as well as it's linked tab
Function NavSync_Off(ByRef objSrcTab, ByRef objDstTab, ByRef objSrcTabCmd, ByRef objDstTabCmd, ByRef objSrcTabVar, ByRef objDstTabVar, ByRef strSyncLossProfile, ByRef strSyncLostAction, ByRef strSyncRecAction, ByRef syncRecFormatOk)
   Dim updateDstFlag
   Dim objDlg
   'logMsg("NavSync_Off()")

   Call getSyncLossActions(strSyncLossProfile, strSyncLostAction, strSyncRecAction, syncRecFormatOk)
   updateDstFlag = False
   If (objSrcTab.vars.Exists("navsync")) Then
      If (getLinkedTab(objSrcTab, objDstTab)) Then
         If (objDstTab.vars.Exists("navsync")) Then
            objDstTabCmd.SetSourceTab(objDstTab)
            Set objDstTabVar = objDstTab.vars("navsync")
            If Not (objDstTabVar("SyncState")) And (strSyncRecAction <> "") Then
               objDstTabCmd.RunCommand(strSyncRecAction)
            End If
            updateDstFlag = True
         End If
      End If
      Set objSrcTabVar = objSrcTab.vars("navsync")
      If Not (objSrcTabVar("SyncState")) And (strSyncRecAction <> "") Then
         objSrcTabCmd.RunCommand(strSyncRecAction)
      End If
      objSrcTab.vars.Delete("navsync")
      objSrcTabCmd.UpdateToggle
      If (updateDstFlag) Then
         objDstTab.vars.Delete("navsync")
         objDstTabCmd.UpdateToggle
      End If
   Else
      Set objDlg = objSrcTab.Dlg
      objDlg.icon = "warning"
      objDlg.message = "NavSync NOT enabled for current SrcTab!"
      objDlg.title = "NavSync:  OFF Failure"
      logMsg("----------------------------------------------------------------")
      logMsg("NavSync NOT enabled for current SrcTab!")
      objDlg.Show
      Set objDlg = Nothing
   End If

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Turn NavSync ON by updating the environment variables for the tab ID passed to this function as well as the destination tab
Function NavSync_On(ByRef commandData, ByRef objSrcTab, ByRef objDstTab, ByRef objSrcTabCmd, ByRef objDstTabCmd, ByRef objSrcTabVarNew, ByRef objDstTabVarNew, ByRef syncRecFormatOk)
   Dim DOpusFactory, objDlg
   Dim SlaveTabs
   'logMsg("NavSync_On()")

   ' FUTURE ==  Add option to prompt for which tabs to link together (up to four from any four listers) and
   ' assign the tab ID's to a new LinkedTabID 'vector' in the navsync environment variable for each linked tab...

   NavSync_On = False
   If (commandData.func.desttab = 0) Then
      logMsg("  No destination tab is available to sync with")
      Exit Function
   Else
      Set objDstTab = commandData.func.desttab
      objDstTab.Update
      objDstTabCmd.SetSourceTab(objDstTab)
   End If

   SlaveTabs = False
   If (objSrcTab.linktab <> 0) Then
      If (objSrcTab.slavetab) Then SlaveTabs = True
   End If
   If (objDstTab.linktab <> 0) Then
      If (objDstTab.slavetab) Then SlaveTabs = True
   End If
   If (SlaveTabs) Then
      Set objDlg = objSrcTab.Dlg
      objDlg.icon = "warning"
      objDlg.message = "NavSync is not supported" & vbCrLf & "with SLAVE tabs enabled!"
      objDlg.title = "NavSync:  ON Failure"
      logMsg("----------------------------------------------------------------")
      logMsg("NavSync is not supported with SLAVE tabs enabled!")
      objDlg.Show
      Set objDstTab = Nothing
      Set objDlg = Nothing
      Exit Function
   End If

   NavSync_On = True
   Set DOpusFactory = DOpus.Create

   ' Cleanup all NavSync variables if the current source and destination tabs already have NavSync enabled...
   If (objSrcTab.vars.Exists("navsync")) Then objSrcTab.vars.Delete("navsync")
   If (objDstTab.vars.Exists("navsync")) Then objDstTab.vars.Delete("navsync")

   ' The 'navsync' tab variable being used to control script behavior will be cast as a Map object
   Set objSrcTabVarNew = DOpusFactory.Map("LinkedTabID", objDstTab, "SyncState", True, "InitPath", objSrcTab.path, "SyncPath", objSrcTab.path)
   Set objDstTabVarNew = DOpusFactory.Map("LinkedTabID", objSrcTab, "SyncState", True, "InitPath", objDstTab.path, "SyncPath", objDstTab.path)

   ' Set the new environnment variables
   objSrcTab.vars.Set("navsync") = objSrcTabVarNew
   objDstTab.vars.Set("navsync") = objDstTabVarNew

   ' If the SYNC_OK_FORMAT option is enabled, set the 'navsync_ok' format for both tabs
   If (syncRecFormatOk) Then
      objSrcTabCmd.RunCommand("set format navsync_ok")
      objDstTabCmd.RunCommand("set format navsync_ok")
   End If

   ' Force an update to the toggle status of any NavSync buttons
   objSrcTabCmd.UpdateToggle
   objDstTabCmd.UpdateToggle

   logStats(objSrcTab)

   Set DOpusFactory = Nothing

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Reset both the SrcTab and DstTab to their original InitPath
Function NavSync_Reset(ByRef objSrcTab, ByRef objDstTab, ByRef objSrcTabCmd, ByRef objDstTabCmd, ByRef objSrcTabVar, ByRef objDstTabVar, ByRef objSrcTabVarNew, ByRef objDstTabVarNew, ByRef strSyncLossProfile, ByRef strSyncLostAction, ByRef strSyncRecAction, ByRef syncRecFormatOk)
   Dim objDlg
   Const vbQuote = """"
   'logMsg("NavSync_Reset()")

   'logMsg("SrcTab-> InitPath:  " & objSrcTabVar("InitPath"))
   'logMsg("DstTab-> InitPath:  " & objDstTabVar("InitPath"))

   'objSrcTabCmd.RunCommand("Go " & vbQuote & objSrcTabVar("InitPath") & vbQuote & " NOSCRIPT")
   objDstTabCmd.RunCommand("Go " & vbQuote & objDstTabVar("InitPath") & vbQuote & " NOSCRIPT")

   If (objDstTabCmd.results.result = 1) Then
      logMsg("DstTab path RESET OK!")
      If Not (objSrcTabVar("SyncState")) And (strSyncRecAction <> "") Then objSrcTabCmd.RunCommand(strSyncRecAction)
      If Not (objDstTabVar("SyncState")) And (strSyncRecAction <> "") Then objDstTabCmd.RunCommand(strSyncRecAction)
      ' If the SYNC_OK_FORMAT option is enabled, set the 'navsync_ok' format for both tabs
      If (syncRecFormatOk) Then
         objSrcTabCmd.RunCommand("set format navsync_ok")
         objDstTabCmd.RunCommand("set format navsync_ok")
      End If
      objSrcTabVarNew("SyncState") = True
      objDstTabVarNew("SyncState") = True
      objSrcTabVarNew("SyncPath") = objSrcTabVar("InitPath")
      objDstTabVarNew("SyncPath") = objDstTabVar("InitPath")

      objSrcTab.vars.Set("navsync") = objSrcTabVarNew
      objDstTab.vars.Set("navsync") = objDstTabVarNew
   Else
      logMsg("DstTab path RESET FAILED! Disabling NavSync...")
      Call NavSync_Off(objSrcTab, objDstTab, objSrcTabCmd, objDstTabCmd, objSrcTabVar, objDstTabVar, strSyncLossProfile, strSyncLostAction, strSyncRecAction, syncRecFormatOk)
   End If

   ' Force an update to the toggle status of any NavSync buttons
   objSrcTabCmd.UpdateToggle
   objDstTabCmd.UpdateToggle

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Resync the DstTab with the SrcTab by creating any missing directory paths in the destination
Function NavSync_Resync(ByRef strNewPath, ByRef objSrcTab, ByRef objDstTab, ByRef objSrcTabCmd, ByRef objDstTabCmd, ByRef objSrcTabVar, ByRef objDstTabVar, ByRef objSrcTabVarNew, ByRef objDstTabVarNew, ByRef strNewFullPath, ByRef strNewRelPath, ByRef strSyncLostAction, ByRef strSyncRecAction, ByRef syncRecFormatOk)
   Dim objNewRelPathParts
   Dim strNewRelPathPart
   Const vbQuote = """"
   'logMsg("NavSync_Resync()")

   logMsg("  Path does NOT exist - RESYNC:  " & DOpus.FSUtil.DisplayName(strNewFullPath))

   objDstTabCmd.RunCommand("CreateFolder " & vbQuote & strNewFullPath & vbQuote & " NOUPDATESETTINGS READAUTO=no")
   ' Depending if the DstTab createfolder worked or not, run the sync-loss recovery action
   ' if necessary BEFORE changing folders. This is the scripts way of hedging bets that if
   ' the folder create worked, so should reading the same path...
   If (objDstTabCmd.results.result = 1) Then
      If Not (objDstTabVar("SyncState")) And (strSyncRecAction <> "") Then objDstTabCmd.RunCommand(strSyncRecAction)
   End If

   ' Call getRelativePath() with an offset of 1, to strip out the trailing slash from the path extraction result since we will SPLIT on a slash char below
   Call getRelativePath(objDstTab.path, strNewFullPath, strNewRelPath, 1, "  ")
   objNewRelPathParts = Split(strNewRelPath, "/")

   strNewRelPathPart = ""
   For Each strNewRelPathPart in objNewRelPathParts
      objDstTabCmd.RunCommand("Go " & vbQuote & objDstTab.path & "/" & strNewRelPathPart & vbQuote & " NOSCRIPT")
      ' The check above is just for visualy cleanliness with sync-loss actions. Only update environment if the path change did in fact actually work...
      If (objDstTabCmd.results.result = 1) Then
         logMsg("DstTab path RESYNC OK!")
         If Not (objSrcTabVar("SyncState")) And (strSyncRecAction <> "") Then objSrcTabCmd.RunCommand(strSyncRecAction)
         If Not (objDstTabVar("SyncState")) And (strSyncRecAction <> "") Then objDstTabCmd.RunCommand(strSyncRecAction)
         ' If the SYNC_OK_FORMAT option is enabled, set the 'navsync_ok' format for both tabs
         If (syncRecFormatOk) Then
            objSrcTabCmd.RunCommand("set format navsync_ok")
            objDstTabCmd.RunCommand("set format navsync_ok")
         End If
         objSrcTabVarNew("SyncState") = True
         objDstTabVarNew("SyncState") = True
         objSrcTabVarNew("SyncPath") = strNewPath
         objDstTabVarNew("SyncPath") = objDstTab.path & "/" & strNewRelPathPart

         objSrcTab.vars.Set("navsync") = objSrcTabVarNew
         objDstTab.vars.Set("navsync") = objDstTabVarNew
         objDstTab.Update
      Else
         If (strSyncLostAction <> "") Then objDstTabCmd.RunCommand(strSyncLostAction)
         logMsg("DstTab path RESYNC FAILED!")
         'objSrcTabVarNew("SyncState") = False
         objDstTabVarNew("SyncState") = False

         objSrcTab.vars.Set("navsync") = objSrcTabVarNew
         objDstTab.vars.Set("navsync") = objDstTabVarNew
         Exit For
      End If
   Next

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Subroutine to log all the relevant tab environment info used by navsync
Sub logStats(ByRef objSrcTab)
   Dim objDstTab, objSrcTabVar, objDstTabVar
   logMsg("----------------------------------------------------------------")

   objSrcTab.Update
   If (objSrcTab.vars.Exists("navsync")) Then
      Set objSrcTabVar = objSrcTab.vars("navsync")
      logMsg("SOURCE TAB:             " & objSrcTab)
      logMsg("Path:                   " & DOpus.FSUtil.DisplayName(objSrcTab.path))
      logMsg("----------------------------------------------------------------")
      logMsg("SrcTab-> LinkedTabID:   " & objSrcTabVar("LinkedTabID"))
      logMsg("SrcTab-> SyncState:     " & objSrcTabVar("SyncState"))
      logMsg("SrcTab-> InitPath:      " & DOpus.FSUtil.DisplayName(objSrcTabVar("InitPath")))
      logMsg("SrcTab-> SyncPath:      " & DOpus.FSUtil.DisplayName(objSrcTabVar("SyncPath")))
      logMsg("----------------------------------------------------------------")
   Else
      logMsg("SOURCE TAB: NAVSYNC NOT ENABLED")
      logMsg("----------------------------------------------------------------")
      Exit Sub
   End If

   If Not (getLinkedTab(objSrcTab, objDstTab)) Then
      logMsg("  The tab which the current Srctab was synced with is no longer open")
      Set objSrcTabVar = Nothing
      Exit Sub
   Else
      If (objDstTab.vars.Exists("navsync")) Then
         Set objDstTabVar = objDstTab.vars("navsync")
         logMsg("DESTINATION TAB:        " & objDstTab)
         logMsg("Path:                   " & DOpus.FSUtil.DisplayName(objDstTab.path))
         logMsg("----------------------------------------------------------------")
         logMsg("DstTab-> LinkedTabID:   " & objDstTabVar("LinkedTabID"))
         logMsg("DstTab-> SyncState:     " & objDstTabVar("SyncState"))
         logMsg("DstTab-> InitPath:      " & DOpus.FSUtil.DisplayName(objDstTabVar("InitPath")))
         logMsg("DstTab-> SyncPath:      " & DOpus.FSUtil.DisplayName(objDstTabVar("SyncPath")))
      Else
         logMsg("DESTINATION TAB: NAVSYNC NOT ENABLED")
      End If
   End If

   Set objSrcTabVar = Nothing
   Set objDstTabVar = Nothing
   Set objDstTab = Nothing

End Sub


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Subroutine to allow toggling of the DEBUG variables to control whether logging is performed or not
Sub logMsg(ByRef message)

   If ((Script.config.DEBUG) Or (SCRIPT_DEBUG)) Then DOpus.Output(message)

End Sub
